"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SlickPaginationComponent = void 0;
const common_1 = require("@slickgrid-universal/common");
const binding_1 = require("@slickgrid-universal/binding");
class SlickPaginationComponent {
    constructor(paginationService, pubSubService, sharedService, translaterService) {
        this.paginationService = paginationService;
        this.pubSubService = pubSubService;
        this.sharedService = sharedService;
        this.translaterService = translaterService;
        this._enableTranslate = false;
        this._subscriptions = [];
        this.firstButtonClasses = '';
        this.lastButtonClasses = '';
        this.prevButtonClasses = '';
        this.nextButtonClasses = '';
        // text translations (handled by i18n or by custom locale)
        this.textItemsPerPage = 'items per page';
        this.textItems = 'items';
        this.textOf = 'of';
        this.textPage = 'Page';
        this._bindingHelper = new binding_1.BindingHelper();
        this._bindingEventService = new binding_1.BindingEventService();
        this._bindingHelper.querySelectorPrefix = `.${this.gridUid} `;
        this.currentPagination = this.paginationService.getFullPagination();
        this._enableTranslate = this.gridOptions?.enableTranslate ?? false;
        if (this._enableTranslate && (!this.translaterService || !this.translaterService.translate)) {
            throw new Error('[Slickgrid-Universal] requires a Translate Service to be installed and configured when the grid option "enableTranslate" is enabled.');
        }
        this.translatePaginationTexts();
        if (this._enableTranslate && this.pubSubService?.subscribe) {
            const translateEventName = this.translaterService?.eventName ?? 'onLanguageChange';
            this._subscriptions.push(this.pubSubService.subscribe(translateEventName, () => this.translatePaginationTexts()));
        }
        // Anytime the pagination is initialized or has changes,
        // we'll copy the data into a local object so that we can add binding to this local object
        this._subscriptions.push(this.pubSubService.subscribe('onPaginationRefreshed', paginationChanges => {
            for (const key of Object.keys(paginationChanges)) {
                this.currentPagination[key] = paginationChanges[key];
            }
            this.updatePageButtonsUsability();
            if (this._spanInfoFromToElm?.style) {
                this._spanInfoFromToElm.style.display = (this.currentPagination.totalItems === 0) ? 'none' : '';
            }
        }), this.pubSubService.subscribe('onPaginationSetCursorBased', () => {
            this.dispose(); // recreate pagination component, probably only used for GraphQL E2E tests
            this.renderPagination(this._gridParentContainerElm);
        }));
    }
    get availablePageSizes() {
        return this.paginationService.availablePageSizes || [];
    }
    get dataFrom() {
        return this.paginationService.dataFrom;
    }
    get dataTo() {
        return this.paginationService.dataTo;
    }
    get itemsPerPage() {
        return this.paginationService.itemsPerPage;
    }
    set itemsPerPage(count) {
        this.paginationService.changeItemPerPage(count);
    }
    get pageCount() {
        return this.paginationService.pageCount;
    }
    get pageNumber() {
        return this.paginationService.pageNumber;
    }
    get grid() {
        return this.sharedService.slickGrid;
    }
    get gridOptions() {
        return this.sharedService.gridOptions;
    }
    get gridUid() {
        return this.grid?.getUID() ?? '';
    }
    get locales() {
        // get locales provided by user in main file or else use default English locales via the Constants
        return this.gridOptions?.locales ?? common_1.Constants.locales;
    }
    get totalItems() {
        return this.paginationService.totalItems;
    }
    get isLeftPaginationDisabled() {
        return this.pageNumber === 1 || this.totalItems === 0;
    }
    get isRightPaginationDisabled() {
        return this.pageNumber === this.pageCount || this.totalItems === 0;
    }
    dispose() {
        // also dispose of all Subscriptions
        this.pubSubService.unsubscribeAll(this._subscriptions);
        this._bindingEventService.unbindAll();
        this._bindingHelper.dispose();
        this._paginationElement.remove();
    }
    renderPagination(gridParentContainerElm) {
        this._gridParentContainerElm = gridParentContainerElm;
        const paginationElm = this.createPaginationContainer();
        const divNavContainerElm = (0, common_1.createDomElement)('div', { className: 'slick-pagination-nav' });
        // left nav
        const leftNavElm = (0, common_1.createDomElement)('nav', { ariaLabel: 'Page navigation' });
        const leftUlElm = (0, common_1.createDomElement)('ul', { className: 'pagination' });
        this._seekFirstElm = (0, common_1.createDomElement)('li', { className: 'page-item seek-first' }, leftUlElm);
        this._seekFirstElm.appendChild((0, common_1.createDomElement)('a', { className: 'page-link icon-seek-first', ariaLabel: 'First Page', role: 'button' }));
        this._seekPrevElm = (0, common_1.createDomElement)('li', { className: 'page-item seek-prev' }, leftUlElm);
        this._seekPrevElm.appendChild((0, common_1.createDomElement)('a', { className: 'page-link icon-seek-prev', ariaLabel: 'Previous Page', role: 'button' }));
        leftNavElm.appendChild(leftUlElm);
        const pageNumberSectionElm = this.createPageNumberSection();
        // right nav
        const rightNavElm = (0, common_1.createDomElement)('nav', { ariaLabel: 'Page navigation' });
        const rightUlElm = (0, common_1.createDomElement)('ul', { className: 'pagination' });
        this._seekNextElm = (0, common_1.createDomElement)('li', { className: 'page-item seek-next' }, rightUlElm);
        this._seekNextElm.appendChild((0, common_1.createDomElement)('a', { className: 'page-link icon-seek-next', ariaLabel: 'Next Page', role: 'button' }));
        this._seekEndElm = (0, common_1.createDomElement)('li', { className: 'page-item seek-end' }, rightUlElm);
        this._seekEndElm.appendChild((0, common_1.createDomElement)('a', { className: 'page-link icon-seek-end', ariaLabel: 'Last Page', role: 'button' }));
        rightNavElm.appendChild(rightUlElm);
        // append both navs to container
        paginationElm.appendChild(divNavContainerElm);
        divNavContainerElm.appendChild(leftNavElm);
        divNavContainerElm.appendChild(pageNumberSectionElm);
        divNavContainerElm.appendChild(rightNavElm);
        const paginationSettingsElm = this.createPaginationSettingsSection();
        paginationElm.appendChild(divNavContainerElm);
        paginationElm.appendChild(paginationSettingsElm);
        this._paginationElement.appendChild(paginationElm);
        if (gridParentContainerElm?.appendChild && this._paginationElement) {
            gridParentContainerElm.appendChild(this._paginationElement);
        }
        this.renderPageSizes();
        this.addBindings();
        this.addEventListeners();
        this.updatePageButtonsUsability();
    }
    /** Render and fill the Page Sizes <select> element */
    renderPageSizes() {
        if (this._itemPerPageElm && Array.isArray(this.availablePageSizes)) {
            for (const option of this.availablePageSizes) {
                this._itemPerPageElm.appendChild((0, common_1.createDomElement)('option', { value: `${option}`, text: `${option}` }));
            }
        }
    }
    /** Add some DOM Element bindings */
    addBindings() {
        this._bindingHelper.addElementBinding(this, 'firstButtonClasses', 'li.page-item.seek-first', 'className');
        this._bindingHelper.addElementBinding(this, 'prevButtonClasses', 'li.page-item.seek-prev', 'className');
        this._bindingHelper.addElementBinding(this, 'lastButtonClasses', 'li.page-item.seek-end', 'className');
        this._bindingHelper.addElementBinding(this, 'nextButtonClasses', 'li.page-item.seek-next', 'className');
        this._bindingHelper.addElementBinding(this.currentPagination, 'dataFrom', 'span.item-from', 'textContent');
        this._bindingHelper.addElementBinding(this.currentPagination, 'dataTo', 'span.item-to', 'textContent');
        this._bindingHelper.addElementBinding(this.currentPagination, 'totalItems', 'span.total-items', 'textContent');
        this._bindingHelper.addElementBinding(this.currentPagination, 'pageCount', 'span.page-count', 'textContent');
        this._bindingHelper.addElementBinding(this.currentPagination, 'pageSize', 'select.items-per-page', 'value');
        this.paginationService.isCursorBased
            ? this._bindingHelper.addElementBinding(this.currentPagination, 'pageNumber', 'span.page-number', 'textContent')
            : this._bindingHelper.addElementBinding(this.currentPagination, 'pageNumber', 'input.page-number', 'value', 'change', this.changeToCurrentPage.bind(this));
        // locale text changes
        this._bindingHelper.addElementBinding(this, 'textItems', 'span.text-items', 'textContent');
        this._bindingHelper.addElementBinding(this, 'textItemsPerPage', 'span.text-item-per-page', 'textContent');
        this._bindingHelper.addElementBinding(this, 'textOf', 'span.text-of', 'textContent');
        this._bindingHelper.addElementBinding(this, 'textPage', 'span.text-page', 'textContent');
    }
    /** Add some DOM Element event listeners */
    addEventListeners() {
        this._bindingEventService.bind(this._seekFirstElm, 'click', this.changeToFirstPage.bind(this));
        this._bindingEventService.bind(this._seekEndElm, 'click', this.changeToLastPage.bind(this));
        this._bindingEventService.bind(this._seekNextElm, 'click', this.changeToNextPage.bind(this));
        this._bindingEventService.bind(this._seekPrevElm, 'click', this.changeToPreviousPage.bind(this));
        this._bindingEventService.bind(this._itemPerPageElm, 'change', this.updateItemsPerPage.bind(this));
    }
    changeToFirstPage(event) {
        if (!this.isLeftPaginationDisabled) {
            this.paginationService.goToFirstPage(event);
        }
    }
    changeToLastPage(event) {
        if (!this.isRightPaginationDisabled) {
            this.paginationService.goToLastPage(event);
        }
    }
    changeToNextPage(event) {
        if (!this.isRightPaginationDisabled) {
            this.paginationService.goToNextPage(event);
        }
    }
    changeToPreviousPage(event) {
        if (!this.isLeftPaginationDisabled) {
            this.paginationService.goToPreviousPage(event);
        }
    }
    changeToCurrentPage(pageNumber) {
        this.paginationService.goToPageNumber(+pageNumber);
    }
    updateItemsPerPage(event) {
        this.itemsPerPage = +(event?.target?.value ?? 0);
    }
    /** Translate all the texts shown in the UI, use ngx-translate service when available or custom locales when service is null */
    translatePaginationTexts() {
        if (this._enableTranslate && this.translaterService?.translate) {
            const translationPrefix = (0, common_1.getTranslationPrefix)(this.gridOptions);
            this.textItemsPerPage = this.translaterService.translate(`${translationPrefix}ITEMS_PER_PAGE`);
            this.textItems = this.translaterService.translate(`${translationPrefix}ITEMS`);
            this.textOf = this.translaterService.translate(`${translationPrefix}OF`);
            this.textPage = this.translaterService.translate(`${translationPrefix}PAGE`);
        }
        else if (this.locales) {
            this.textItemsPerPage = this.locales.TEXT_ITEMS_PER_PAGE || 'TEXT_ITEMS_PER_PAGE';
            this.textItems = this.locales.TEXT_ITEMS || 'TEXT_ITEMS';
            this.textOf = this.locales.TEXT_OF || 'TEXT_OF';
            this.textPage = this.locales.TEXT_PAGE || 'TEXT_PAGE';
        }
    }
    // --
    // protected functions
    // --------------------
    /** Create the Pagination Container */
    createPaginationContainer() {
        const paginationContainerElm = (0, common_1.createDomElement)('div', {
            id: 'pager', className: `slick-pagination-container ${this.gridUid} pager`,
            style: { width: '100%' },
        });
        const paginationElm = (0, common_1.createDomElement)('div', { className: 'slick-pagination' });
        paginationContainerElm.appendChild(paginationElm);
        this._paginationElement = paginationContainerElm; // keep internal ref
        return paginationElm;
    }
    createPageNumberSection() {
        const divElm = (0, common_1.createDomElement)('div', { className: 'slick-page-number' });
        (0, common_1.createDomElement)('span', { className: 'text-page', textContent: 'Page' }, divElm);
        divElm.appendChild(document.createTextNode(' '));
        if (this.paginationService.isCursorBased) {
            // cursor based navigation cannot jump to an arbitrary page. Simply display current page number.
            (0, common_1.createDomElement)('span', {
                className: 'page-number',
                ariaLabel: 'Page Number',
                dataset: { test: 'page-number-label' },
                textContent: '1',
            }, divElm);
        }
        else {
            // offset based navigation can jump to any page. Allow editing of current page number.
            (0, common_1.createDomElement)('input', {
                type: 'text',
                className: 'form-control page-number',
                ariaLabel: 'Page Number',
                value: '1', size: 1,
                dataset: { test: 'page-number-input' },
            }, divElm);
        }
        divElm.appendChild(document.createTextNode(' '));
        (0, common_1.createDomElement)('span', { className: 'text-of', textContent: 'of' }, divElm);
        divElm.appendChild(document.createTextNode(' '));
        (0, common_1.createDomElement)('span', { className: 'page-count', dataset: { test: 'page-count' } }, divElm);
        return divElm;
    }
    createPaginationSettingsSection() {
        const spanContainerElm = (0, common_1.createDomElement)('span', { className: 'slick-pagination-settings' });
        this._itemPerPageElm = (0, common_1.createDomElement)('select', { id: 'items-per-page-label', ariaLabel: 'Items per Page', className: 'items-per-page' }, spanContainerElm);
        spanContainerElm.appendChild(document.createTextNode(' '));
        (0, common_1.createDomElement)('span', { className: 'text-item-per-page', textContent: 'items per page' }, spanContainerElm);
        spanContainerElm.appendChild(document.createTextNode(', '));
        const spanPaginationCount = (0, common_1.createDomElement)('span', { className: 'slick-pagination-count' }, spanContainerElm);
        this._spanInfoFromToElm = (0, common_1.createDomElement)('span', { className: 'page-info-from-to' }, spanPaginationCount);
        (0, common_1.createDomElement)('span', { className: 'item-from', ariaLabel: 'Page Item From', dataset: { test: 'item-from' } }, this._spanInfoFromToElm);
        this._spanInfoFromToElm.appendChild(document.createTextNode('-'));
        (0, common_1.createDomElement)('span', { className: 'item-to', ariaLabel: 'Page Item To', dataset: { test: 'item-to' } }, this._spanInfoFromToElm);
        this._spanInfoFromToElm.appendChild(document.createTextNode(' '));
        (0, common_1.createDomElement)('span', { className: 'text-of', textContent: 'of' }, this._spanInfoFromToElm);
        this._spanInfoFromToElm.appendChild(document.createTextNode(' '));
        const spanInfoTotalElm = (0, common_1.createDomElement)('span', { className: 'page-info-total-items' }, spanPaginationCount);
        (0, common_1.createDomElement)('span', { className: 'total-items', ariaLabel: 'Total Items', dataset: { test: 'total-items' } }, spanInfoTotalElm);
        spanInfoTotalElm.appendChild(document.createTextNode(' '));
        (0, common_1.createDomElement)('span', { className: 'text-items', textContent: 'items' }, spanInfoTotalElm);
        spanInfoTotalElm.appendChild(document.createTextNode(' '));
        return spanContainerElm;
    }
    updatePageButtonsUsability() {
        this.firstButtonClasses = this.isLeftPaginationDisabled ? 'page-item seek-first disabled' : 'page-item seek-first';
        this.prevButtonClasses = this.isLeftPaginationDisabled ? 'page-item seek-prev disabled' : 'page-item seek-prev';
        this.lastButtonClasses = this.isRightPaginationDisabled ? 'page-item seek-end disabled' : 'page-item seek-end';
        this.nextButtonClasses = this.isRightPaginationDisabled ? 'page-item seek-next disabled' : 'page-item seek-next';
    }
}
exports.SlickPaginationComponent = SlickPaginationComponent;
//# sourceMappingURL=slick-pagination.component.js.map